
/*
 * Isomorphic Smart GWT web presentation layer
 * Copyright 2000 and beyond Isomorphic Software, Inc.
 *
 * OWNERSHIP NOTICE
 * Isomorphic Software owns and reserves all rights not expressly granted in this source code,
 * including all intellectual property rights to the structure, sequence, and format of this code
 * and to all designs, interfaces, algorithms, schema, protocols, and inventions expressed herein.
 *
 *  If you have any questions, please email <sourcecode@isomorphic.com>.
 *
 *  This entire comment must accompany any portion of Isomorphic Software source code that is
 *  copied or moved from this file.
 */package com.smartgwt.client.docs.serverds;

import com.smartgwt.client.types.*;
import com.smartgwt.client.data.DSRequest;
import com.smartgwt.client.widgets.form.validator.Validator;
import com.smartgwt.client.widgets.form.fields.FormItem;
import java.util.List;
import java.util.Map;


/**
 * <b>These settings are documentation of the settings in the {@link com.smartgwt.client.docs.serverds.DsDescriptor 'XML DataSource descriptor file'}.</b>
 * <p>
 * A DataSource is data-provider-independant description of a set of objects that will be loaded, edited and saved within
 * the user interface of your application. <P> Each DataSource consists of a list of {@link
 * com.smartgwt.client.docs.serverds.DataSource#fields 'fields'} that make up a DataSource <code>record</code>, along with
 * {@link com.smartgwt.client.docs.serverds.DataSourceField#type 'field types'},  {@link
 * com.smartgwt.client.docs.serverds.DataSourceField#validators 'validation rules'},  {@link
 * com.smartgwt.client.docs.serverds.DataSourceField#foreignKey 'relationships'} to other DataSources, and other metadata.
 * <P> The abstract object description provided by a DataSource is easily mapped to a variety of backend object models and
 * storage schemes.  The following table shows analogous terminology across systems. <table border=1 class="normal"> <tr>  
 * <td>Isomorphic Smart GWT</td>   <td>Relational Database</td>   <td>Enterprise Java Beans (EJB)</td>  
 * <td>Entity/Relationship Modelling</td>   <td>OO/UML</td>   <td>XML Schema/WSDL</td>   <td>LDAP</td> </tr><tr>  
 * <td>DataSource</td>   <td>Table</td>   <td>EJB class</td>   <td>Entity</td>   <td>Class</td>   <td>Element Schema
 * (ComplexType)</td>   <td>Objectclass</td> </tr><tr>   <td>Record</td>   <td>Row</td>   <td>EJB instance</td>  
 * <td>Entity instance</td>   <td>Class instance/Object</td>   <td>Element instance (ComplexType)</td>   <td>Entry</td>
 * </tr><tr>   <td>Field</td>   <td>Column</td>   <td>Property</td>   <td>Attribute</td>   <td>Property/Attribute</td>  
 * <td>Attribute or Element (SimpleType)</td>   <td>Attribute</td> </tr></table> <P> DataSources can be {@link
 * com.smartgwt.client.docs.DataSourceDeclaration 'declared'} in either JavaScript or XML format, and can also be {@link
 * com.smartgwt.client.docs.MetadataImport 'imported'} from existing metadata formats, including XML Schema. <P> <i>Data
 * Binding</i> is the process by which {@link com.smartgwt.client.widgets.DataBoundComponent} can automatically configure
 * themselves for viewing, editing and saving data described by DataSources.  DataBinding is covered in the
 * ${isc.DocUtils.linkForDocNode('QuickStartGuide', 'QuickStart Guide')}, Chapter 6, <i>Data Binding</i>. <P> {@link
 * com.smartgwt.client.docs.ClientServerIntegration 'Data Integration'} is the process by which a DataSource can be
 * connected to server systems such as SQL DataBases, Java Object models, WSDL web services and other data providers.  Data
 * Integration comes in two variants: client-side and server-side.  {@link com.smartgwt.client.docs.ServerDataIntegration
 * 'Server-side integration'} uses the Smart GWT Java-based server to connect to data represented by Java Objects or
 * JDBC-accessible databases.  {@link com.smartgwt.client.docs.ClientDataIntegration 'Client-side integration'} connects
 * Smart GWT DataSources to XML, JSON or other formats accessible via HTTP. <P> DataSources have a concept of {@link
 * com.smartgwt.client.docs.DataSourceOperations '4 core operations'} ("fetch", "add", "update" and "remove") that can be
 * performed on the set of objects represented by a DataSource.  Once a DataSource has been integrated with your data
 * store, databinding-capable  UI components can leverage the 4 core DataSource operations to provide many complete user
 * interactions without the need to configure how each individual component loads and saves data. <P> These interactions
 * include {@link com.smartgwt.client.widgets.grid.ListGrid}, {@link com.smartgwt.client.widgets.tree.TreeGrid},  {@link
 * com.smartgwt.client.widgets.viewer.DetailViewer}, {@link com.smartgwt.client.widgets.form.DynamicForm}-based  {@link
 * com.smartgwt.client.widgets.form.DynamicForm#editRecord} and {@link
 * com.smartgwt.client.widgets.form.DynamicForm#saveData}, grid-based {@link
 * com.smartgwt.client.widgets.grid.ListGrid#getCanEdit 'editing'} and {@link
 * com.smartgwt.client.widgets.grid.ListGrid#getSaveByCell 'saving'}, and custom interactions provided by
 * ${isc.DocUtils.linkForExampleId('patternReuse')} custom databinding-capable components.
 */
public class DataSource {

    /**
     * For a DataSource that inherits {@link com.smartgwt.client.docs.serverds.DataSource#fields fields} from another
     * DataSource  (via {@link com.smartgwt.client.docs.serverds.DataSource#inheritsFrom inheritsFrom}), indicates that only
     * the fields listed in this DataSource should be shown.  All other inherited parent fields will be marked "hidden:true".
     *
     * <p>Default value is null
     */
    public Boolean showLocalFieldsOnly;
             
    /**
     * Sets the strategy this DataSource uses to translate Java enumerated types (objects of type enum) to and from Javascript.
     *  This property is only applicable if you are using the Smart GWT server
     *
     * <p>Default value is null
     */
    public EnumTranslateStrategy enumTranslateStrategy;
             
    /**
     * Transport to use for all operations on this DataSource. Defaults to {@link
     * com.smartgwt.client.rpc.RPCManager#defaultTransport}.  This would typically only be set to enable "scriptInclude"
     * transport for contacting {@link com.smartgwt.client.docs.serverds.DataSource#dataFormat 'JSON'} web services hosted on
     * servers other than the origin server. <p> When using the "scriptInclude" transport, be sure to set {@link
     * com.smartgwt.client.docs.serverds.DataSource#callbackParam callbackParam} or {@link
     * com.smartgwt.client.docs.serverds.OperationBinding#callbackParam callbackParam} to match the name of the query parameter
     * name expected by your JSON service provider.
     *
     * <p>Default value is RPCManager.defaultTransport
     */
    public RPCTransport dataTransport;
             
    /**
     * Indicates the format to be used for HTTP requests and responses when fulfilling DSRequests (eg, when {@link
     * com.smartgwt.client.data.DataSource#fetchData} is called).
     *
     * <p>Default value is "iscServer"
     */
    public DSDataFormat dataFormat;

    /**
     * Designates a field of {@link com.smartgwt.client.types.FieldType}:"image" as the field to use when rendering a record as
     * an image, for example, in a {@link com.smartgwt.client.widgets.tile.TileGrid}. <p> For example, for a DataSource of
     * employees, a "photo" field of type "image" should be designated as the iconField. <p> If not explicitly set, iconField
     * looks for fields named "picture", "thumbnail", "icon", "image" and "img", in that order, and will use any of these
     * fields as the iconField if it exists and has type "image".  <P> To avoid any field being used as the iconField, set
     * iconField to <code>null</code>.
     *
     * <p>Default value is see below
     */
    public String iconField;

    /**
     * Whether RPCRequests sent by this DataSource should enable  {@link com.smartgwt.client.rpc.RPCRequest#getShowPrompt
     * showPrompt} if it's unset.
     *
     * <p>Default value is true
     */
    public Boolean showPrompt;

    /**
     * Whether to act as a "client-only" DataSource, which has no permanent storage and never contacts the server, instead
     * using a set of test data to respond to requests in the same manner as a server-based DataSource might. <P> See {@link
     * com.smartgwt.client.docs.ClientOnlyDataSources 'this discussion'} for ways to populate a client-only DataSource with
     * test data.
     *
     * <p>Default value is false
     */
    public Boolean clientOnly;

    /**
     * Very advanced: for servers that do not support paging, and must return large numbers of XML records in one HTTP
     * response, Smart GWT breaks up the processing of the response in order to avoid the "script running slowly" dialog
     * appearing for an end user. <P> If you have a relatively small number of records with a great deal of properties or
     * subojects on each record, and you have not set {@link com.smartgwt.client.docs.serverds.DataSource#dropExtraFields
     * dropExtraFields} to eliminate unused data, and you see the "script running slowly" dialog, you may need to set this
     * number lower.
     *
     * <p>Default value is 150
     */
    public int resultBatchSize;

    /**
     * User-visible name for this DataSource. <P> For example, for the supplyItem DataSource, "Supply Item". <P> Defaults to
     * <code>dataSource.ID</code>.
     *
     * <p>Default value is dataSource.ID
     */
    public String title;

    /**
     * The name of the property this DataSource uses for constant name when translating Java enumerated types to and from
     * Javascript, if the {@link com.smartgwt.client.types.EnumTranslateStrategy} is set to "bean".  Defaults to "_constant" if
     * not set. <p>  This property is only applicable if you are using the Smart GWT server
     *
     * <p>Default value is null
     */
    public String enumConstantProperty;

    /**
     * Allows you to specify an arbitrary suffix string to apply to all json format responses  sent from the server to this
     * application.<br> The inclusion of such a suffix ensures your code is not directly executable outside of your
     * application, as a preventitive measure against <a href='http://www.google.com/search?q=javascript+hijacking'
     * onclick="window.open('http://www.google.com/search?q=javascript+hijacking');return false;">javascript hijacking</a>.
     * <br> Only applies to responses formatted as json objects. Does not apply to responses returned via scriptInclude type
     * transport.
     *
     * <p>Default value is null
     */
    public String jsonSuffix;

    /**
     * The name of the property this DataSource uses for ordinal number when translating Java enumerated types to and from
     * Javascript, if the {@link com.smartgwt.client.types.EnumTranslateStrategy} is set to "bean".  Defaults to "_ordinal" if
     * not set. <p>  This property is only applicable if you are using the Smart GWT server
     *
     * <p>Default value is null
     */
    public String enumOrdinalProperty;

    /**
     * This property has different meanings depending on the {@link com.smartgwt.client.docs.serverds.DataSource#serverType
     * 'serverType'}: <p> <b>For DataSources with serverType "hibernate"</b><br> The name of the Java bean or POJO class that
     * is mapped in Hibernate.  This will typically  be the fully-qualified class name - eg <code>com.foo.MyClass</code> - but
     * it may be the  simple class name - just <code>MyClass</code> - or it may be some other value.  It all  depends on how
     * your classes have been mapped in Hibernate. <p> Note:  If you are intending to use Hibernate as a data-access layer
     * only,  you do not need to create Hibernate mappings or Java objects: Smart GWT will generate everything it needs on the
     * fly. <p> <b>For DataSources with serverType "sql"</b><br> If set, results from the database will be used to create one
     * instance of the indicated Java bean per database row.  Otherwise a Map is used to represent each row retrieved from SQL.
     * <P> With this feature active, a DSResponse from this DataSource will contain a Collection of instances of the indicated
     * <code>beanClassName</code>, available via DSResponse.getData(). This creates a couple of possibilities: <P> <dl> <dt>Add
     * business logic for derived properties, such as computed formulas</dt> <dd>For example, declare a DataSourceField named
     * "revenueProjection".  By default this field will call getRevenueProjection() on your bean to retrieve the value to send
     * to the client.  Your implementation of getRevenueProjection() could apply some kind of formula to other values loaded
     * from the database.</dd> <dt>Call business logic on retrieved beans via DMI</dt> <dd>By adding a {@link
     * com.smartgwt.client..DMI} method that calls DSRequest.execute() to retrieve a DSResponse, you have an opportunity to
     * call business logic methods on the beans representing each row affected by the DSRequest.  For example, notify a related
     * BPEL process of changes to certain fields.</dd> </dl> <P> By using <code>beanClassName</code> on a specific {@link
     * com.smartgwt.client.data.OperationBinding}, you can: <ul> <li>Use a bean to represent your data only when it matters;
     * for example, avoid the overhead     of using a bean for "fetch" operations, but do use a bean for "update" operations so
     * that you can execute relevant business logic after the update completes.</li> <li>Skip the use of beans for complex
     * reporting queries that produce results unrelated to     your persistent object model.  Set beanClassName to blank ("")
     * on a specific     operationBinding to override DataSource.beanClassName for that specific operation.</li> <li>For SQL
     * joins that produce additional data fields, use a special, operation-specific     bean that represents a join of multiple
     * entities and contains business logic specific     to that joined dataset</li> </ul> <P> Note that setting
     * <code>beanClassName</code> has no effect on the DSRequest.  However, {@link com.smartgwt.client..DMI} has a built-in
     * facility for populating a bean with the inbound {@link com.smartgwt.client.data.DSRequest#getData data} - just declare
     * the bean as a method argument. <p> <b>For DataSources with serverType "generic"</b><br> {@link
     * com.smartgwt.client.docs.VisualBuilder 'Visual Builder'} sets this property when it creates a generic  DataSource using
     * the Javabean Wizard.  It has no built-in server-side effects.
     *
     * <p>Default value is null
     */
    public String beanClassName;

    /**
     * User-visible plural name for this DataSource. <P> For example, for the supplyItem DataSource, "Supply Items". <P>
     * Defaults to (<code>dataSource.title</code> or <code>dataSource.ID</code>) + "s".
     *
     * <p>Default value is dataSource.ID
     */
    public String pluralTitle;

    /**
     * For a DataSource that inherits {@link com.smartgwt.client.docs.serverds.DataSource#fields fields} from another
     * DataSource  (via {@link com.smartgwt.client.docs.serverds.DataSource#inheritsFrom inheritsFrom}), indicates that the
     * parent's field order should be used instead of the order of the fields as declared in this DataSource.  New fields, if
     * any, are placed at the end.
     *
     * <p>Default value is null
     */
    public Boolean useParentFieldOrder;

    /**
     * For DataSources using the {@link com.smartgwt.client.docs.SqlDataSource 'Smart GWT SQL engine'} for persistence, what
     * database table name to use.  The default is to use the DataSource ID as the table name.
     *
     * <p>Default value is null
     */
    public String tableName;

    /**
     * Allows you to specify an arbitrary prefix string to apply to all json format responses  sent from the server to this
     * application.<br> The inclusion of such a prefix ensures your code is not directly executable outside of your
     * application, as a preventitive measure against <a href='http://www.google.com/search?q=javascript+hijacking'
     * onclick="window.open('http://www.google.com/search?q=javascript+hijacking');return false;">javascript hijacking</a>.
     * <br> Only applies to responses formatted as json objects. Does not apply to responses returned via scriptInclude type
     * transport.<br> Note: If the prefix / suffix served by your backend is not a constant, you can use  {@link
     * com.smartgwt.client.data.OperationBinding#getDataFormat 'dataFormat:"custom"'} instead and explicitly parse the prefix
     * out as part of {@link com.smartgwt.client.data.DataSource#transformResponse}.
     *
     * <p>Default value is null
     */
    public String jsonPrefix;

    /**
     * For DataSources using the {@link com.smartgwt.client.docs.SqlDataSource 'Smart GWT SQL engine'} for persistence, which
     * database configuration to use.  Database configurations can be created using the "Databases" tab in the Developer
     * Console.  If unset, the default database configuration is used (which is also settable using the "Databases" tab).
     *
     * <p>Default value is null
     */
    public String dbName;
             
    /**
     * For a DataSource stored in .xml format on the ISC server, indicates what server-side connector to use to execute
     * requests, that is, what happens if you call dsRequest.execute() in server code.
     *
     * <p>Default value is "generic"
     */
    public DSServerType serverType;

    /**
     * Applies only to dataFormat: "json".  Specifies the name of the query parameter that tells your JSON service what
     * function to call as part of the response.
     *
     * <p>Default value is "callback"
     */
    public String callbackParam;

    /**
     * Analagous to {@link com.smartgwt.client.docs.serverds.DataSource#dropExtraFields dropExtraFields}, for data sent to the
     * server.  Setting this attribute to false ensures that for any records in the data object, only fields that correspond to
     * declared dataSource fields will be present on the dsRequest data object passed to {@link
     * com.smartgwt.client.data.DataSource#transformRequest} and ultimately sent to the server.
     *
     * <p>Default value is true
     */
    public Boolean sendExtraFields;

    /**
     * Default URL to contact to fulfill all DSRequests.  Can also be set on a per-operationType basis via {@link
     * com.smartgwt.client.docs.serverds.OperationBinding#dataURL dataURL}. <P> NOTE: Best practice is to use the same
     * <code>dataURL</code> for all DataSources which fulfill DSRequests via the server-side RPCManager API.  Otherwise,
     * cross-DataSource {@link com.smartgwt.client.rpc.RPCManager#startQueue} will not be possible.
     *
     * <p>Default value is null
     */
    public String dataURL;

    /**
     * Whether to attempt validation on the client at all for this DataSource.  If unset (the default), client-side validation
     * is enabled. <p> Disabling client-side validation entirely is a good way to test server-side validation.
     *
     * <p>Default value is null
     */
    public Boolean useLocalValidators;

    /**
     * For a DataSource with serverType: "sql" or serverType: "hibernate", automatically derive the dataSource's schema (field
     * definitions) from the SQL table specified in  {@link com.smartgwt.client.docs.serverds.DataSource#tableName tableName}. 
     * This causes Smart GWT to create a "super" DataSource, which this dataSource then automatically {@link
     * com.smartgwt.client.docs.serverds.DataSource#inheritsFrom 'inheritsFrom'}.  This allows you to  override auto-derived
     * schema as required. <p> This property is only applicable if you are using the Smart GWT server.
     *
     * <p>Default value is null
     */
    public Boolean autoDeriveSchema;

    /**
     * If set, titles are automatically derived from {@link com.smartgwt.client.docs.serverds.DataSourceField#name
     * 'field.name'} for any  field that does not have a {@link com.smartgwt.client.docs.serverds.DataSourceField#title
     * 'field.title'} and is not marked {@link com.smartgwt.client.docs.serverds.DataSourceField#hidden 'hidden'}:true, by
     * calling the method {@link com.smartgwt.client.data.DataSource#getAutoTitle}.
     *
     * <p>Default value is true
     */
    public Boolean autoDeriveTitles;

    /**
     * Specifies the fully-qualified name of the DataSource subclass that should be instantiated server-side for this
     * dataSource. This property allows you to write and use custom DataSource  subclasses on the server. <p> One reason you
     * might wish to do this would be to override the validate() method to provide  some arbitrary custom validation (such as
     * complex database lookups, validation embedded in  legacy applications, etc).  It is also possible - though obviously a
     * more substantial task - to override the execute() method in your custom DataSource.  This is one way of creating  a
     * completely customized DataSource implementation. <p> <b>Note:</b> If you use this property, you are responsible for
     * making sure that it refers to a valid server-side class that extends
     * <code>com.isomorphic.datasource.BasicDataSource</code>. If your implementation relies on methods or state only present
     * in certain specialized  subclasses of DataSource (for example, you want the normal behavior and features of a 
     * HibernateDataSource, but with a specialized validate() method), then you shoud extend the  subclass rather than the base
     * class.
     *
     * <p>Default value is null
     */
    public String serverConstructor;

    /**
     * For a DataSource derived from WSDL or XML schema, the XML namespace this schema belongs to.  This is a read-only
     * attribute automatically present on DataSources returned from {@link com.smartgwt.client..SchemaSet#getSchema} and {@link
     * com.smartgwt.client.data.WebService#getSchema}.
     *
     * <p>Default value is null
     */
    public String schemaNamespace;

    /**
     * Best field to use for a user-visible title for an individual record from this dataSource. <p> For example, for a
     * DataSource of employees, a "full name" field would probably most clearly label an employee record. <p> If not explicitly
     * set, titleField looks for fields named "title", "label", "name", and "id" in that order.  If a field exists with one of
     * those names, it becomes the titleField.  If not, then the first field is designated as the titleField.
     *
     * <p>Default value is see below
     */
    public String titleField;

    /**
     * For dataSources of {@link com.smartgwt.client.docs.serverds.DataSource#serverType 'serverType'} "sql", determines
     * whether we qualify column names with table names in any SQL we generate.  This property can be overridden on specific
     * operationBindings.
     *
     * <p>Default value is true
     */
    public Boolean qualifyColumnNames;
             
    /**
     * Decides under what conditions the {@link com.smartgwt.client.data.ResultSet} cache should be dropped when the {@link
     * com.smartgwt.client.data.ResultSet#getCriteria criteria} changes.
     *
     * <p>Default value is "dropOnShortening"
     */
    public CriteriaPolicy criteriaPolicy;

    /**
     * Indicates that for server responses, for any data being interpreted as DataSource records,  only data that corresponds
     * to declared fields should be retained; any extra fields should be discarded. <P> For {@link
     * com.smartgwt.client.docs.serverds.DataSource#dataFormat 'JSON'} data, this means extra properties in selected objects
     * are dropped. <P> By default, for DMI DSResponses, DSResponse.data is filtered on the server to just the set of fields
     * defined on the DataSource.  This type of filtering can also be enabled for non-DMI DSResponses (see the overview in
     * {@link com.smartgwt.client..DMI}).  Setting this property to <code>false</code> disables this filtering for this
     * DataSource only.  This setting overrides the configuration in <code>server.properties</code>.  This setting can be
     * overridden by {@link com.smartgwt.client.docs.serverds.ServerObject#dropExtraFields dropExtraFields}.
     *
     * <p>Default value is null
     */
    public Boolean dropExtraFields;

    /**
     * [A] If set to true, both client and server-side advanced filtering used by Smart GWT will follow&#010 SQL99 behavior for
     * dealing with NULL values, which is often counter-intuitive to users.&#010 Specifically, when a field has NULL value, all
     * of the following expressions are false:&#010 <pre>&#010    field == "someValue"  (normally false)&#010    field !=
     * "someValue"  (normally true)&#010    not (field == "someValue")   (normally true)&#010    not (field != "someValue")  
     * (normally false)&#010 </pre>
     *
     * <p>Default value is false
     */
    public Boolean strictSQLFiltering;

    /**
     * See {@link com.smartgwt.client.docs.serverds.OperationBinding#recordXPath recordXPath}.  <code>recordXPath</code> can be
     * specified directly on the DataSource for a simple read-only DataSource only capable of "fetch" operations.
     *
     * <p>Default value is null
     */
    public String recordXPath;

    /**
     * fieldName for a field in the dataSource expected to contain an explicit array of child nodes.&#010 Enables loading a
     * databound tree as a hierachical data structure, rather than a flat list of&#010 nodes linked by foreignKey.<br>&#010
     * Note this is an alternative to setting {@link com.smartgwt.client.docs.serverds.DataSourceField#childrenProperty
     * childrenProperty} directly on&#010 the childrenField object.<P>&#010 By default the children field will be assumed to be
     * {@link com.smartgwt.client.docs.serverds.DataSourceField#multiple 'multiple'},&#010 for XML databinding. This implies
     * that child data should be delivered in the format:&#010 <pre>&#010      &lt;childrenFieldName&gt;&#010          &lt;item
     * name="firstChild" ...&gt;&#010          &lt;item name="secondChild" ...&gt;&#010      &lt;/childrenFieldName&gt;&#010
     * </pre>&#010 However data may also be delivered as a direct list of <code>childrenFieldName</code> &#010 elements:&#010
     * <pre>&#010      &lt;childrenFieldName name="firstChild" ...&gt;&#010      &lt;childrenFieldName name="secondChild"
     * ...&gt;&#010 </pre>&#010 If you want to return your data in this format, you will need to explicitly set &#010
     * <code>multiple</code> to false in the appropriate dataSource field definition.
     *
     * <p>Default value is null
     */
    public String childrenField;

    /**
     * Like {@link com.smartgwt.client.widgets.DataBoundComponent#getUseFlatFields useFlatFields}, but applies to all DataBound
     * components that bind to this DataSource.
     *
     * <p>Default value is null
     */
    public Boolean useFlatFields;

    /**
     * If set, the DataSource will ensure that it never uses a cached HTTP response, even if the server marks the response as
     * cacheable. <P> Note that this does not disable caching at higher levels in the framework, for example, the caching
     * performed by {@link com.smartgwt.client.data.ResultSet}.
     *
     * <p>Default value is true
     */
    public Boolean preventHTTPCaching;

    /**
     * For an XML DataSource, URN of the WebService to use to invoke operations.  This URN comes from the "targetNamespace"
     * attribute of the &lt;wsdl:definitions&gt; element in a WSDL (Web Service Description Language) document, and serves as
     * the unique identifier of the service. <P> Having loaded a WebService using {@link
     * com.smartgwt.client.data.XMLTools#loadWSDL}, setting <code>serviceNamespace</code> combined with specifying  {@link
     * com.smartgwt.client.data.OperationBinding} that set {@link com.smartgwt.client.data.OperationBinding#getWsOperation
     * wsOperation} will cause a DataSource to invoke web service operations to fulfill DataSource requests ({@link
     * com.smartgwt.client.data.DSRequest}). <P> Setting <code>serviceNamespace</code> also defaults {@link
     * com.smartgwt.client.docs.serverds.DataSource#dataURL 'dataURL'} to the service's location, {@link
     * com.smartgwt.client.docs.serverds.DataSource#dataFormat 'dataFormat'} to "xml" and {@link
     * com.smartgwt.client.docs.serverds.OperationBinding#dataProtocol 'dataProtocol'} to "soap".
     *
     * <p>Default value is null
     */
    public String serviceNamespace;

    /**
     * Tag name to use when serializing to XML.  If unspecified, the <code>dataSource.ID</code> will be used.
     *
     * <p>Default value is null
     */
    public String tagName;

    /**
     * When true, indicates that this DataSource supports multi-level sorting.
     *
     * <p>Default value is true
     */
    public Boolean canMultiSort;


    /**
     * For a client-only DataSource, a set of records to use as a dataset, specified as an Array of JavaScript Objects. <P> See
     * {@link com.smartgwt.client.docs.ClientOnlyDataSources 'this discussion'} for ways to populate a client-only DataSource
     * with test data.
     *
     */
    public Object[] testData;

    /**
     * Class for ResultSets used by this datasource.  If null, defaults to using {@link com.smartgwt.client.data.ResultSet}.
     * <P> This can be set to a custom subclass of ResultSet that, for example, hangs onto to extra information necessary for
     * integration with web services.
     *
     */
    public String resultSetClass;

    /**
     * The list of fields that compose records from this DataSource. <P> Each DataSource field can have type, user-visible
     * title, validators, and other metadata attached.
     *
     */
    public DataSourceField[] fields;

    /**
     * Namespaces definitions to add to the root element of outbound XML messages sent to a web&#010 service, as a mapping from
     * namespace prefix to namespace URI.&#010 <P>&#010 The default value is:&#010 <pre>&#010   globalNamespaces : {&#010
     * xsi: "http://www.w3.org/2001/XMLSchema-instance",&#010      xsd: "http://www.w3.org/2001/XMLSchema"&#010   },&#010
     * </pre>&#010 This default value allows the use of the xsi:type and xsi:nil attributes without further&#010
     * declarations.&#010 <P>&#010 Note that some web services will only accept specific revisions of the XML Schema URI.&#010
     * If xsi-namespaced attributes seem to be ignored by an older webservice, try the URI&#010
     * "http://www.w3.org/1999/XMLSchema-instance" instead.
     *
     */
    public Map globalNamespaces;

    /**
     * Optional array of OperationBindings, which provide instructions to the DataSource about how each&#010 DSOperation is to
     * be performed.&#010 <P>&#010 When using the Smart GWT Server, OperationBindings are specified in your DataSource&#010
     * descriptor (.ds.xml file) and control server-side behavior such as what Java object to route&#010 DSRequest to ({@link
     * com.smartgwt.client.docs.serverds.OperationBinding#serverObject serverObject}) or customizations to SQL / HQL queries&#010
     * ({@link com.smartgwt.client.docs.serverds.OperationBinding#customSQL customSQL} and {@link
     * com.smartgwt.client.docs.serverds.OperationBinding#customHQL customHQL}).  See the &#010
     * ${isc.DocUtils.linkForExampleId('javaDataIntegration', 'Java Integration samples')}.&#010 <P>&#010 For DataSources bound
     * to WSDL-described web services using&#010 {@link com.smartgwt.client.docs.serverds.DataSource#serviceNamespace
     * serviceNamespace}, OperationBindings are used to bind each DataSource&#010 {@link
     * com.smartgwt.client.docs.serverds.OperationBinding#operationType 'operationType'} to an&#010 {@link
     * com.smartgwt.client.docs.serverds.OperationBinding#wsOperation 'operation'} of a WSDL-described &#010 {@link
     * com.smartgwt.client.data.WebService}, so that a DataSource can both fetch and save data to a web&#010 service.&#010
     * <P>&#010 For example, this code accomplishes part of the binding to the &#010 <a
     * href='http://www.google.com/search?q=sforce+partner+wsdl'
     * onclick="window.open('http://www.google.com/search?q=sforce+partner+wsdl');return false;">SalesForce partner web
     * services</a>&#010 <pre>&#010 isc.DataSource.create({&#010    serviceNamespace : "urn:partner.soap.sforce.com",&#010
     * operationBindings : [&#010        { operationType:"fetch", wsOperation:"query", recordName: "sObject" },&#010        {
     * operationType:"update", wsOperation:"update", recordName: "SaveResult" },&#010        { operationType:"add",
     * wsOperation:"create", recordName: "SaveResult" },&#010        { operationType:"remove", wsOperation:"delete",
     * recordName: "DeleteResult" }&#010    ],&#010    ...&#010 }); &#010 </pre>&#010 NOTE: additional code is required to
     * handle authentication and other details, see the&#010 complete code in
     * isomorphicSDK/examples/databinding/SalesForce.&#010 <P>&#010 For DataSources that contact non-WSDL-described XML or JSON
     * services, OperationBindings can&#010 be used to separately configure the URL, HTTP method, input and output processing
     * for each&#010 operationType.  This makes it possible to fetch JSON data from one URL for the "fetch"&#010 operationType
     * and save to a web service for the "update" operationType, while appearing as a&#010 single integrated DataSource to a
     * {@link com.smartgwt.client.widgets.DataBoundComponent} such as an&#010 {@link
     * com.smartgwt.client.widgets.grid.ListGrid#setCanEdit 'editable ListGrid'}.&#010 <P>&#010 If no operationBinding is
     * defined for a given DataSource operation, all of the properties&#010 which are valid on the operationBinding are checked
     * for on the DataSource itself.  &#010 <P>&#010 This also means that for a read-only DataSource, that is, a DataSource
     * only capable of fetch&#010 operations, operationBindings need not be specified, and instead all operationBinding&#010
     * properties can be set on the DataSource itself.
     *
     */
    public OperationBinding[] operationBindings;

    /**
     * Additional properties to pass through to the {@link com.smartgwt.client.data.DSRequest}s made by this DataSource. <p>
     * These properties are applied before {@link com.smartgwt.client.data.DataSource#transformRequest} is called.
     *
     */
    public DSRequest requestProperties;

    /**
     * Class for ResultTrees used by this datasource.  If null, defaults to using ResultTree. <P>
     * This can be set to a custom subclass of ResultTree that, for example, hangs on to extra information necessary for
     * integration with web services.
     *
     */
    public String resultTreeClass;

    /**
     * ID of another DataSource this DataSource inherits its DataSource fields from.<br><br>Local fields (fields defined
     * in this DataSource) are added to inherited fields to form the full set of fields. Fields with the same name are
     * merged in the same way that databound component fields are merged with DataSource fields.<br><br>The default
     * order of the combined fields is new local fields first (including any fields present in the parent DataSource
     * which the local DataSource re-declares), then parent fields.  You can set {@link
     * com.smartgwt.client.docs.serverds.DataSource#useParentFieldOrder useParentFieldOrder} to instead use the
     * parent's field order, with new local fields appearing last.<br><br>
     * <p/>
     * You can set {@link com.smartgwt.client.docs.serverds.DataSource#showLocalFieldsOnly}  to
     * have all non-local fields hidden.<br><br>Note that &lt;b&gt;only fields are inherited&lt;/b&gt; - other
     * properties such as dataURL and dataFormat are not.  You can use ordinary inheritance, that is, creating a
     * subclass of DataSource, in order to share properties such as dataURL across a series of DataSources that also
     * inherit fields from each other via <code>inheritsFrom</code> <br><br> This feature can be used for
     * creating a customized view (eg, only certain fields shown) which will be used bymultiple databound
     * components.adding presentation-specific attributes to metadata that has been automatically derived from {@link
     * com.smartgwt.client.data.XMLTools#loadXMLSchema(String, com.smartgwt.client.data.XSDLoadCallback)} or other metadata formats modelling
     * object subclassing and extension in server-side code and storage systems modelling relational database joins, and
     * the equivalents in other systems creating hooks for others to customize your application in a maintainable way.
     * For example, if you have a dataSource &quot;employee&quot;, you can create a
     * dataSource&quot;customizedEmployee&quot; which inherits from &quot;employee&quot; but does not initially define
     * anyfields, and bind all databound components to&quot;customizedEmployee&quot;. Customizations of fields
     * (including appearance changes, fieldorder, new fields, hiding of fields, and custom validation rules) can be
     * added to&quot;customizedEmployee&quot;, so that they are kept separtely from the original field data andhave the
     * best possible chance of working with future versions of the &quot;employee&quot;dataSource.&lt;/ul&gt;
     *
     */
    public String inheritsFrom;

    /**
     * For Direct Method Invocation (DMI) binding, declares the ServerObject to use as the default target for all {@link
     * com.smartgwt.client.docs.serverds.DataSource#operationBindings}.  Specifying this attribute in an
     * XML DataSource stored on the server enables DMI for this DataSource.
     *
     */
    public ServerObject serverObject;
    

}
