
/*
 * Isomorphic Smart GWT web presentation layer
 * Copyright 2000 and beyond Isomorphic Software, Inc.
 *
 * OWNERSHIP NOTICE
 * Isomorphic Software owns and reserves all rights not expressly granted in this source code,
 * including all intellectual property rights to the structure, sequence, and format of this code
 * and to all designs, interfaces, algorithms, schema, protocols, and inventions expressed herein.
 *
 *  If you have any questions, please email <sourcecode@isomorphic.com>.
 *
 *  This entire comment must accompany any portion of Isomorphic Software source code that is
 *  copied or moved from this file.
 */package com.smartgwt.client.docs.serverds;

import com.smartgwt.client.types.*;
import com.smartgwt.client.data.DSRequest;
import com.smartgwt.client.widgets.form.validator.Validator;
import com.smartgwt.client.widgets.form.fields.FormItem;
import java.util.List;
import java.util.Map;


/**
 * <b>These settings are documentation of the settings in the {@link com.smartgwt.client.docs.serverds.DsDescriptor 'XML DataSource descriptor file'}.</b>
 * <p>
 * Metadata about a DataSourceField, including its type and validators.
 */
public class DataSourceField {

    /**
     * Dictates whether the data in this field be exported.  Explicitly setting  <i>canExport</i> to false overrides the
     * setting on any component-fields, such as {@link com.smartgwt.client.widgets.grid.ListGridField#getCanExport 'ListGrid
     * fields'}.
     *
     * <p>Default value is null
     */
    public Boolean canExport;

    /**
     * For use in {@link com.smartgwt.client.docs.ComponentSchema}, a field {@link
     * com.smartgwt.client.docs.serverds.DataSource#inheritsFrom 'inherited'} from another schema can be redeclared with this
     * property set in order to indicate that the property should not be used. <P> This is primarily used to influence {@link
     * com.smartgwt.client.docs.VisualBuilder}.  For simple type properties, this avoids the property appearing in the
     * Component Editor.   <P> For fields that hold subcomponents, this prevents inappropriate drag and drop.  For example, a
     * custom class called <code>MyDialog</code> may automatically create a series of children, and not allow arbitrary other
     * children to be added.  In this case, the inherited property {@link com.smartgwt.client.widgets.Canvas#getChildren
     * children} should be marked inapplicable in order to prevent arbitrary components being dropped onto a
     * <code>MyDialog</code> instance.
     *
     * <p>Default value is null
     */
    public Boolean inapplicable;

    /**
     * Declares that this field holds values that can be matched to values from another DataSource&#010 field, to create a
     * relationship between records from different DataSources or even records&#010 within the same DataSource.&#010 <p>&#010
     * The format of <code>foreignKey</code> is&#010 <code>&lt;dataSourceId&gt;.&lt;fieldName&gt;</code>.<br>&#010 For a
     * foreignKey within the same dataSource, you can omit the <code>dataSourceId</code>&#010 and just specify
     * <code>&lt;fieldName&gt;</code>.<br>&#010 For example, to create a tree relationship within a DataSource:&#010 <pre>&#010
     * DataSource.create({&#010     ID:"supplyItem",&#010     fields : [&#010       {name:"itemId", type:"sequence",
     * primaryKey:true},&#010       {name:"parentId", type:"integer", foreignKey:"itemId"},&#010       ...&#010     ]&#010  
     * });&#010 </pre>&#010 <P>&#010 <code>foreignKey</code> declarations also allow other automatic behaviors by&#010 {@link
     * com.smartgwt.client.widgets.DataBoundComponent}, such as {@link
     * com.smartgwt.client.widgets.grid.ListGrid#fetchRelatedData}.
     *
     * <p>Default value is false
     */
    public String foreignKey;

    /**
     * See {@link com.smartgwt.client.docs.serverds.DataSourceField#javaClass javaClass} - when auto-populating of Java Beans /
     * POJOs based on  inbound DSRequest data, for a field of Map type, <code>javaKeyClass</code> can be used to specify a
     * particular concrete class for the map keys.  If not specified, and a concrete type cannot be discovered using Java
     * Reflection, <code>java.lang.Object</code> is used. Note that <code>javaKeyClass</code> take precedence over generics if
     * both are used.
     *
     * <p>Default value is null
     */
    public String javaKeyClass;

    /**
     * Indicates that this field should always be Array-valued.  If the value derived from&#010 {@link
     * com.smartgwt.client.docs.serverds.DataSource#dataFormat 'XML or JSON data'} is singular, it will be wrapped in an
     * Array.&#010 <P>&#010 Specifically for XML serialization and deserialization, <code>multiple:true</code> behaves&#010
     * similarly to the &#010 <a href='http://www.google.com/search?hl=en&q=soap+array'
     * onclick="window.open('http://www.google.com/search?hl=en&q=soap+array');return false;">SOAP array idiom</a>, that
     * is,&#010 there will be a "wrapper element" named after the field name, whose contents will be several&#010 elements of
     * the specified {@link com.smartgwt.client.docs.serverds.DataSourceField#type 'field.type'}.  &#010 <P>&#010 For example,
     * {@link com.smartgwt.client.widgets.layout.Layout#getMembers members} is declared with <code>type:"Canvas",&#010
     * multiple:true</code>.  The correct XML format is thus:&#010 <pre>&#010 &lt;VLayout&gt;&#010     &lt;members&gt;&#010    
     * &lt;Canvas ID="myCanvas" ... /&gt;&#010         &lt;ListGrid ID="myGrid" .../&gt;&#010         &lt;Toolstrip
     * ID="myToolStrip" ... /&gt;&#010     &lt;/members&gt;&#010 &lt;/VLayout&gt;&#010 </pre>&#010 <P>&#010 See {@link
     * com.smartgwt.client.docs.serverds.DataSourceField#childTagName childTagName} for customizing the tagName used for
     * subelements.
     *
     * <p>Default value is null
     */
    public Boolean multiple;

    /**
     * For use in {@link com.smartgwt.client.docs.ComponentSchema}, indicates what group to place the property in when editing
     * in Visual Builder.
     *
     * <p>Default value is null
     */
    public String group;

    /**
     * Should the user be able to filter data by this field. Effects whether this field will show up in dataBoundComponents
     * with UI for filtering data.
     *
     * <p>Default value is null
     */
    public Boolean canFilter;

    /**
     * For a field of type:"imageFile", indicates whether to stream the image and display it inline or to display the View and
     * Download icons.
     *
     * <p>Default value is null
     */
    public Boolean showFileInline;

    /**
     * Whether this field can ever be edited by the user.  If set to <code>false</code>, no DataBound component will ever try
     * to offer an editing interface for this field.
     *
     * <p>Default value is null
     */
    public Boolean canEdit;

    /**
     * For a DataSource with {@link com.smartgwt.client.docs.serverds.DataSource#serverType 'serverType:"sql"'} with a field of
     * type "sequence", the name of the SQL sequence that should be used when inserting new records into this table. <P> Note
     * that this is never required for SQL tables that are generated from Smart GWT DataSources (a default sequence name of
     * tableName + "_" + columnName is chosen), and is  never required for databases where inserting null into a sequence
     * column is sufficient (MySQL, SQL Server, DB2 and others).   <P> You would only need to set sequenceName if you are
     * integrating with a pre-existing table stored in a database where the sequence must be named for insertion to work
     * (Oracle, Postgres) OR you are trying to use the same sequence across multiple DataSources.
     *
     * <p>Default value is null
     */
    public String sequenceName;

    /**
     * Whether this field should be considered a "detail" field by a {@link com.smartgwt.client.widgets.DataBoundComponent}.
     * <P> Detail fields won't be shown by default in a DataBoundComponent where  {@link
     * com.smartgwt.client.widgets.DataBoundComponent#getShowDetailFields showDetailFields} is false.  This allows for some
     * DataBound components, like a {@link com.smartgwt.client.widgets.grid.ListGrid}, to show a summary view of records which
     * displays only  the most commonly viewed fields by default, while other DataBoundComponents, like a {@link
     * com.smartgwt.client.widgets.viewer.DetailViewer}, show all fields by default. <P> In addition, the {@link
     * com.smartgwt.client.widgets.form.fields.FormItem#showIf} property is supported in multiple components for conditional
     * visibility - see for example {@link com.smartgwt.client.widgets.grid.ListGridField#showIf} and {@link
     * com.smartgwt.client.widgets.form.fields.FormItem#showIf}).
     *
     * <p>Default value is false
     */
    public Boolean detail;

    /**
     * Indicates that {@link com.smartgwt.client.data.DataSource#xmlSerialize} should serialize this value as an XML attribute.
     * <P> Note this does not need to be declared in order for DataSource records to be derived from XML data: a field will be
     * populated with either an attribute or subelement with matching name.
     *
     * <p>Default value is null
     */
    public Boolean xmlAttribute;

    /**
     * For fields of a type that are represented by binary data initially uploaded from a file (currently "binary" and
     * "imageFile", see {@link com.smartgwt.client.types.FieldType}), this sets the maximum  file size allowed, in bytes.
     *
     * <p>Default value is null
     */
    public Integer maxFileSize;

    /**
     * Whether this field holds a value unique across all records in this DataSource.&#010 <p>&#010 A DataSource that can
     * <i>only</i> perform the "fetch" operation does not require a&#010 primaryKey.  If a DataSource allows modification of
     * DataSource records through add, update&#010 and remove DataSource operations, exactly one field must be marked as the
     * primary key.&#010 <P>&#010 Smart GWT requires a primary key value to uniquely identify records when communicating&#010
     * updates or deletions to the server.  There is no requirement that the primaryKey field be&#010 mapped to an actual
     * "primary key" in your object model, web service, or database.  The only&#010 requirement is that values of the
     * primaryKey field be unique for a given browser instance&#010 for the lifetime of the page.  &#010 <P>&#010 If your
     * underlying data model has no primaryKey or has multiple primaryKey(s), you can&#010 generate unique values or combine
     * multiple values into a single field that serves as a&#010 unique key, and declare that field as the single primary key
     * instead.  For example, if your&#010 underlying data model has two primary keys "firstName" and "lastName", you can
     * use&#010 {@link com.smartgwt.client.data.DataSource#transformResponse} to combine them to into a new field
     * "name_key":&#010 <pre>&#010 isc.DataSource.create({&#010    fields:[&#010        {name:"name_key",
     * primaryKey:true},&#010        {name:"firstName"},&#010        {name:"lastName"},&#010        ... other fields ...&#010  
     * ],&#010    transformResponse : function (dsResponse) {&#010        var data = dsResponse.data;&#010        for (var i =
     * 0 ; i &lt; data.length; i++) {&#010            data[i].name_key = data[i].firstName + ":" +&#010                        
     * data[i].lastName;&#010        }&#010    }&#010 });&#010 </pre>&#010 <P>&#010 Here it is assumed that ":" is not a legal
     * character in either the firstName or lastName&#010 field.  Updates to records from this DataSource will then be sent
     * with name_key as the&#010 primary key value, and the server can split the value to obtain the separate key values.&#010
     * <P>&#010 If using ISC's {@link com.smartgwt.client.docs.SqlDataSource 'SQL engine'} and generating SQL tables using
     * the&#010 "Databases" tab of the Developer Console, the table column generated from a primaryKey field&#010 will have a
     * unique constraint applied in the database table.
     *
     * <p>Default value is false
     */
    public Boolean primaryKey;

    /**
     * The table name to use when qualifying the column name for this field during server-side SQL query generation.  Only
     * applicable to "sql" dataSources, and only has an effect when  {@link
     * com.smartgwt.client.docs.serverds.DataSource#qualifyColumnNames qualifyColumnNames} or {@link
     * com.smartgwt.client.docs.serverds.OperationBinding#qualifyColumnNames qualifyColumnNames} is in force. <p> <b>Note:</b>
     * This property is not normally required; the Smart GWT server will ordinarily  qualify all column names correctly without
     * assistance. This property is useful when  qualification can't be done automatically - for example, when you have a
     * custom  {@link com.smartgwt.client.docs.serverds.OperationBinding#tableClause 'tableClause'} that renames tables in the
     * query using aliases.
     *
     * <p>Default value is null
     */
    public String tableName;

    /**
     * Maximum number of characters allowed.  Applicable only to fields of text type.
     *
     * <p>Default value is null
     */
    public Integer length;

    /**
     * Whether this field should be hidden from users by default within a DataBound component. This is generally used for
     * internal IDs and other fields not meaningful to users. <P> See {@link
     * com.smartgwt.client.docs.serverds.DataSourceField#detail detail} for fields that should be hidden in a summary view such
     * as a {@link com.smartgwt.client.widgets.grid.ListGrid}, but still available to the user.
     *
     * <p>Default value is false
     */
    public Boolean hidden;

    /**
     * For use in {@link com.smartgwt.client.docs.ComponentSchema} for fields that contain other components, this flag&#010
     * suppresses auto-construction for subcomponents that appear under this field.&#010 <P>&#010 For example, the {@link
     * com.smartgwt.client.widgets.layout.VLayout} schema sets this for its {@link
     * com.smartgwt.client.widgets.layout.Layout#getMembers 'members'}&#010 property, so that when a VLayout is constructed via
     * XML as follows:&#010 <pre>&#010 &lt;VLayout&gt;&#010     &lt;members&gt;&#010         &lt;ListGrid ID="myGrid"
     * .../&gt;&#010         &lt;Toolstrip ID="myToolStrip" ... /&gt;&#010     &lt;/members&gt;&#010 &lt;/VLayout&gt;&#010
     * </pre>&#010 The ListGrid and ToolStrip do not construct themselves automatically.  Instead, the VLayout&#010 receives
     * the properties of the ListGrid and ToolStrip as ordinary JavaScript Objects, with&#010 the special property
     * <code>_constructor</code> set to the name of the class that should be&#010 constructed.
     *
     * <p>Default value is null
     */
    public Boolean propertiesOnly;

    /**
     * See {@link com.smartgwt.client.docs.serverds.DataSourceField#javaClass javaClass} - when auto-populating of Java Beans /
     * POJOs based on  inbound DSRequest data, for a field of type Collection or Map,  <code>javaCollectionClass</code> can  be
     * used to specify a particular concrete class to use.  If not specified, and a concrete Collection or Map class cannot be
     * discovered using Java Reflection, the following concrete classes are used: <ul> <li> <code>java.util.ArrayList</code> is
     * used for fields of type <code>List</code> <li> <code>java.util.HashSet</code> is used for fields of type
     * <code>Set</code> <li> <code>java.util.LinkedList</code> is used for fields of type <code>Queue</code> <li>
     * <code>org.apache.commons.collections.map.LinkedMap</code> is used for fields of type <code>Map</code> <li>
     * <code>java.util.ArrayList</code> is used for fields that are otherwise of type <code>Collection</code> </ul> Note that
     * this value is used even if the target Collection or Map is declared as a concrete class.  So, for example, if you set
     * <code>javaCollectionClass</code> to  <code>java.util.LinkedList</code> but your setter method accepts a 
     * <code>java.util.ArrayList</code>, you will get a ClassCastException.
     *
     * <p>Default value is null
     */
    public String javaCollectionClass;

    /**
     * For use with the Smart GWT server when populating Java Beans / POJOs based on data contained in a DSRequest,
     * <code>javaClass</code> specifies the fully qualified Java className to be created and passed to the setter for the Java
     * Bean Property with the same name as this field.   <code>javaClass</code> is used both when manually calling
     * DataSource.setProperties() and when auto-populating POJO arguments of a {@link com.smartgwt.client..DMI} method. <P> The
     * Java class to create does not normally have to be specified: Smart GWT will use Java reflection to inspect the type of
     * argument expected by a setter method and will attempt conversion of inbound data to that type.  As described in the
     * documentation for DataTools.setProperties(), this works for almost all typical cases.  However
     * <code>field.javaClass</code> is useful for: <ul> <li> subobject of abstract or interface type: in this case Java
     * Reflection is not sufficient to discover the concrete type that should be instantiated, and <code>javaClass</code>
     * should be set instead.</li> <li> subobject of Collection or Map type, when Java generics are not used or the Collection
     * member type or Map value type is abstract.  When Java generics are used (for example the setter takes an argument is of
     * type Collection&lt;SomePOJO&gt; or Map&lt;KeyType,SomePOJO&gt;, Smart GWT will automatically attempt to convert inbound
     * data to the type of the members of the Collection or values of the Map.  Without generics, <code>javaClass</code> needs
     * to be specified.  Note that <code>javaClass</code> will take precendence over generics if both  are used. Also note that
     * {@link com.smartgwt.client.docs.serverds.DataSourceField#javaCollectionClass javaCollectionClass} can be specified if a
     * particular Collection or Map type is needed, and {@link com.smartgwt.client.docs.serverds.DataSourceField#javaKeyClass
     * javaKeyClass} can be specified for a field of type <code>java.util.Map</code>.</li> </ul>
     *
     * <p>Default value is null
     */
    public String javaClass;

    /**
     * Name for this field.   <p> The field name is also the property in each DataSource record which holds the value for this
     * field. <P> Must be unique across all fields within the DataSource.
     *
     * <p>Default value is null
     */
    public String name;

    /**
     * Optional different field-title used for exports.
     *
     * <p>Default value is null
     */
    public String exportTitle;
             
    /**
     * Type of this field.  Required for all DataSource fields. <P> Field type may imply automatic validators (for example, an
     * integer field cannot accept the value "foo").  Field type also affects the default behaviors of DataBound components,
     * for example, if a field is declared as type "date", components that edit that field will automatically choose a
     * date-editing interface with pop-up date picker.
     *
     * <p>Default value is null
     */
    public FieldType type;

    /**
     * Indicates this field must be non-null in order for a record to pass validation. <P> Note that <code>required</code>
     * should not be set for a server-generated field, such as a sequence, or validation will fail on the client.
     *
     * <p>Default value is null
     */
    public Boolean required;

    /**
     * Whether values in this field can be updated and saved to the dataSource. <P> If set to <code>false</code>, this field
     * will default to being non-editable in standard editing components ({@link com.smartgwt.client.widgets.form.DynamicForm},
     * editable {@link com.smartgwt.client.widgets.grid.ListGrid}), but will be editable when displayed for filtering purposes
     * only (in a {@link com.smartgwt.client.widgets.form.SearchForm} or {@link
     * com.smartgwt.client.widgets.grid.ListGrid#getShowFilterEditor 'ListGrid filter editor'}. <P> Note: if {@link
     * com.smartgwt.client.docs.serverds.DataSourceField#canEdit canEdit} is explicitly specified it will take precedence over
     * this behavior.
     *
     * <p>Default value is null
     */
    public Boolean canSave;

    /**
     * XPath expression used to retrieve the field's value.&#010 <P>&#010 This XPath expression will be evaluated in the scope
     * of the record objects selected by &#010 the {@link com.smartgwt.client.docs.serverds.DataSource#recordXPath
     * recordXPath}.  For XML data &#010 ({@link com.smartgwt.client.docs.serverds.DataSource#dataFormat 'dataFormat:"xml"'})
     * this means a call to&#010 {@link com.smartgwt.client.data.XMLTools#selectString} passing the selected XML element.  For
     * JSON data&#010 ({@link com.smartgwt.client.docs.serverds.DataSource#dataFormat 'dataFormat:"json"'}), this means a call
     * to &#010 {@link com.smartgwt.client.data.XMLTools#selectObjects} passing the selected JSON object.&#010 <P>&#010 In the
     * absence of a <code>valueXPath</code>, for JSON data the value for the field will&#010 be the value of the same-named
     * property in the record object selected by&#010 {@link com.smartgwt.client.docs.serverds.DataSource#recordXPath
     * 'recordXPath'}.  &#010 <P>&#010 For XML data, the value will be the attribute or subelement named after the field
     * name.&#010 For example, for a field "author" on a record element &lt;book&gt;, the following structures&#010 require no
     * valueXPath:&#010 <pre>&#010    &lt;book author="Mark Jones"/&gt;&#010&#010    &lt;book&gt;&#010       
     * &lt;author&gt;Mark Jones&lt;/author&gt;&#010    &lt;/book&gt;&#010 </pre>&#010 <P>&#010 If <code>valueXPath</code> is
     * not required for your field because of the default handling&#010 described above, don't specify it, as it's slightly
     * slower.&#010 <p>&#010 To learn about XPath, try the following search:&#010 <a
     * href="http://www.google.com/search?q=xpath+tutorial" target="_blank"&#010
     * >http://www.google.com/search?q=xpath+tutorial</a>&#010 <P>&#010 <B>Using valueXPath with the Smart GWT server</B>&#010
     * <p>&#010 If you're using the Smart GWT server to return data via the DSResponse object (or&#010 indirectly doing so
     * using DataSource DMI), the valueXPath you specify on the DataSource&#010 fields will be applied to the data you return
     * via the &#010 <a href='http://jakarta.apache.org/commons/jxpath/'
     * onclick="window.open('http://jakarta.apache.org/commons/jxpath/');return false;">JXPath</a> library.&#010 <P>&#010 If
     * you are returning Java Beans as your DSResponse data, normally each dataSource field&#010 receives the value of the
     * same-named Java Bean property, that is, a field "zipCode" is&#010 populated by looking for "getZipCode()" on the objects
     * passed as DSResponse data.  You can&#010 use <code>valueXPath</code> to retrieve properties from subobjects, so long as
     * a chain of&#010 getter methods exists that corresponds to the valueXPath.  For example, a valueXPath of&#010
     * "address/zipCode" expects to call "getAddress()" on the bean(s) passed to&#010 DSResponse.setData(), followed by
     * "getZipCode()" on whatever object "getAddress()" returns.&#010 <P>&#010 When you are saving data, the inbound DSRequest
     * values, available as a Java Map, will use &#010 just dataSource field names as Map keys, not the valueXPath used to
     * derive them.  However,&#010 to achieve bidirectional valueXPath binding, you can use the server-side method&#010
     * dataSource.setProperties() to use the valueXPath when setting properties on your server&#010 object model.  When applied
     * as a setter, an XPath like "address/zipCode" attempts&#010 "getAddress()" followed by "setZipCode()" on the returned
     * object.  JXPath also has some&#010 ability to auto-create intervening objects if they are missing, such as auto-creating
     * an&#010 "address" subobject when applying "address/zipCode" as a valueXPath.&#010 <P>&#010 See the&#010 <a
     * href='http://jakarta.apache.org/commons/jxpath/'
     * onclick="window.open('http://jakarta.apache.org/commons/jxpath/');return false;">JXPath</a> library documentation
     * for&#010 complete details, including other types of server object models supported, such as&#010 server-side XML.
     *
     * <p>Default value is null
     */
    public String valueXPath;

    /**
     * For a field that is {@link com.smartgwt.client.docs.serverds.DataSourceField#multiple 'multiple:"true"'}, controls the
     * name of the XML tag used for each subelement during {@link com.smartgwt.client.data.DataSource#xmlSerialize}. <P> If
     * unset, the default tag name is "value" for a field of simple type, and for a field of DataSource type, is the tagName or
     * ID of the DataSource (as though <code>xmlSerialize()</code> were called on the child DataSource).
     *
     * <p>Default value is null
     */
    public String childTagName;

    /**
     * Used by the {@link com.smartgwt.client.widgets.BatchUploader} to map a field in an upload file to this  dataSourceField.
     * This is only necessary if the dataSourceField's name and title differ  from the name of the field in the upload file
     * (Smart GWT will automatically map upload  fields using the dataSourceField's title, if possible, if it does not get a
     * direct match  on field name).
     *
     * <p>Default value is null
     */
    public String uploadFieldName;

    /**
     * When true, this field can only be used for sorting if the data is entirely client-side.
     *
     * <p>Default value is false
     */
    public Boolean canSortClientOnly;

    /**
     * If set, this property indicates that this dataSourceField is only intended for use as context in a {@link
     * com.smartgwt.client.docs.serverds.OperationBinding#customSQL 'customSQL'} or {@link
     * com.smartgwt.client.docs.serverds.OperationBinding#customHQL 'customHQL'} operationBinding, and should not be considered
     * by the server when generating SQL or  Hibernate queries to fetch or update data.  A common reason for needing to set
     * this flag is where you have a value that you want to send to the server for use in selection criteria, but that value
     * actually comes from another table. <p> Note that this property only applies to users of the Smart GWT server using
     * dataSources of type "sql" or "hibernate".
     *
     * <p>Default value is null
     */
    public Boolean customSQL;

    /**
     * Default user-visible title for this field. <p> This will be picked up by DataBound components and other views over this
     * DataSource.
     *
     * <p>Default value is null
     */
    public String title;

    /**
     * Whether this field should be completely excluded from this dataSource, as if it had never been defined.
     *
     * <p>Default value is false
     */
    public Boolean ignore;

    /**
     * If true, this property indicates that this field will hold an explicit array of child nodes  for the current node. This
     * has the same effect as specifying {@link com.smartgwt.client.docs.serverds.DataSource#childrenField childrenField} to
     * this field's  name.
     *
     * <p>Default value is false
     */
    public Boolean childrenProperty;


    /**
     * For a field that is a foreignKey establishing a tree relationship, what value indicates a root-level node.  Defaults to
     * null. <P> Note that the rootValue may be overridden on a specific ResultTree instance by setting {@link
     * com.smartgwt.client..ResultTree#setRootNode rootNode}, or if the ResultTree is auto-generated by a {@link
     * com.smartgwt.client.widgets.tree.TreeGrid}, by setting {@link com.smartgwt.client.widgets.tree.TreeGrid#setTreeRootValue
     * treeRootValue}. This allows a component to navigate a subtree of the hierarchical data from this dataSource starting at
     * a particular node.
     *
     */
    public Object rootValue;

    /**
     * Validators to be applied to this field. <p> Validators are applied whenever there is an attempt to save changes to a
     * field. <p> For the available set of built-in validators, and how to define a custom validator, see the {@link
     * com.smartgwt.client.widgets.form.validator.Validator} class.
     *
     */
    public Validator[] validators;

    /**
     * List of operators valid on this field.   <P> If not specified, all operators that are valid for the field type are
     * allowed.
     *
     */
    public OperatorId[] validOperators;

    /**
     * Sets the default FormItem to be used whenever this field is edited (whether in a grid, form, or other component). <P> If
     * unset, a FormItem will be automatically chosen based on the type of the field, by the rules explained {@link
     * com.smartgwt.client.types.FormItemType}.
     *
     */
    public FormItem editorType;

    /**
     * A ValueMap is a set of legal values for a field.&#010 <p>&#010 The valueMap can be
     * specified as either an Array of legal values, or as an&#010 Object where each property maps
     * a stored value to a user-displayable value.&#010 <p>&#010 To enforce that a field should be constrained to only the
     * values in the valueMap, either&#010 declare {@link com.smartgwt.client.docs.serverds.DataSourceField#type 'field.type'} as
     * "enum", or use an&#010 {@link com.smartgwt.client.type.ValidatorTypes#isOneOf} with explicitly listed values.
     * Otherwise,&#010 although a normal {@link com.smartgwt.client.widgets.form.fields.SelectItem} control will only allow
     * values from the valueMap to be&#010 entered, other controls such as a {@link
     * com.smartgwt.client.widgets.form.fields.ComboBoxItem} will allow other values &#010 to be entered.&#010 <p>&#010 In XML,
     * a valueMap that specifies only a list of legal values is specified as follows:&#010 <pre>&#010   &lt;valueMap&gt;&#010
     * &lt;value&gt;Pens &amp; Pencils&lt;/value&gt;&#010    &lt;value&gt;Stationery&lt;/value&gt;&#010
     * &lt;value&gt;Computer Products&lt;/value&gt;&#010    &lt;value&gt;Furniture&lt;/value&gt;&#010
     * &lt;value&gt;Misc&lt;/value&gt;&#010   &lt;/valueMap&gt;&#010 </pre>&#010 A ValueMap that specifies stored values mapped
     * to user-visible values is specified as&#010 follows:&#010 <pre>&#010   &lt;valueMap&gt;&#010    &lt;value ID="1"&gt;Pens
     * &amp; Pencils&lt;/value&gt;&#010    &lt;value ID="2"&gt;Stationery&lt;/value&gt;&#010    &lt;value ID="3"&gt;Computer
     * Products&lt;/value&gt;&#010    &lt;value ID="4"&gt;Furniture&lt;/value&gt;&#010    &lt;value
     * ID="5"&gt;Misc&lt;/value&gt;&#010   &lt;/valueMap&gt;&#010 </pre>
     *
     */
    public Map valueMap;


}


