
/*
 * Isomorphic Smart GWT web presentation layer
 * Copyright 2000 and beyond Isomorphic Software, Inc.
 *
 * OWNERSHIP NOTICE
 * Isomorphic Software owns and reserves all rights not expressly granted in this source code,
 * including all intellectual property rights to the structure, sequence, and format of this code
 * and to all designs, interfaces, algorithms, schema, protocols, and inventions expressed herein.
 *
 *  If you have any questions, please email <sourcecode@isomorphic.com>.
 *
 *  This entire comment must accompany any portion of Isomorphic Software source code that is
 *  copied or moved from this file.
 */package com.smartgwt.client.docs.serverds;

import com.smartgwt.client.types.*;
import com.smartgwt.client.data.DSRequest;
import com.smartgwt.client.widgets.form.validator.Validator;
import com.smartgwt.client.widgets.form.fields.FormItem;
import java.util.List;
import java.util.Map;


/**
 * <b>These settings are documentation of the settings in the {@link com.smartgwt.client.docs.serverds.DsDescriptor 'XML DataSource descriptor file'}.</b>
 * <p>
 * An operationBinding tells a DataSource how to execute one of the basic DS operations: fetch, add, update, remove.  See
 * {@link com.smartgwt.client.docs.serverds.DataSource#operationBindings operationBindings}.
 */
public class OperationBinding {
             
    /**
     * Which operationType this operationBinding is for.  This property is only settable on an operationBinding, not a
     * DataSource as a whole.
     *
     * <p>Default value is null
     */
    public DSOperationType operationType;

    /**
     * The style of line-breaks to use in the exported output.  See {@link com.smartgwt.client..LineBreakStyle} for more
     * information.
     *
     * <p>Default value is null
     */
    public String lineBreakStyle;
             
    /**
     * The format in which the data should be exported.  Default is "csv". See {@link com.smartgwt.client.types.ExportFormat}
     * for more information.
     *
     * <p>Default value is "csv"
     */
    public ExportFormat exportAs;

    /**
     * For a DataSource contacting a {@link com.smartgwt.client.docs.serverds.DataSource#serviceNamespace 'WSDL web service'},
     * setting this flag means the DataSource doesn't actually attempt to contact the server but generates a sample response
     * instead, based on the XML Schema of the response message embedded in the WSDL. <P> The spoofed response will include all
     * complexType elements and will fill in appropriate values by type for all simpleType elements, although the spoofed data
     * will not conform to all xs:restriction declarations (eg xs:pattern). <P> Note that if your WSDL does not fully describe
     * the response format (some WSDL services just have a placeholder &lt;xs:any&gt; element), Smart GWT can only produce a
     * partial response.  To use a hand-generated sample response, just save an XML file to disk and use the {@link
     * com.smartgwt.client.docs.serverds.OperationBinding#dataURL dataURL} setting to point to it.
     *
     * <p>Default value is false
     */
    public Boolean spoofResponses;

    /**
     * Applies only to dataFormat: "json".  Specifies the name of the query parameter that tells your JSON service what
     * function to call as part of the response for this operation. <P> Typically set once for the DataSource as a whole via
     * {@link com.smartgwt.client.docs.serverds.DataSource#callbackParam callbackParam}.
     *
     * <p>Default value is "callback"
     */
    public String callbackParam;

    /**
     * If set, every invocation of this operationBinding will invalidate the local cache, forcing a server visit to refresh the
     * data.
     *
     * <p>Default value is null
     */
    public Boolean invalidateCache;

    /**
     * Configures {@link com.smartgwt.client.docs.serverds.DataSource#preventHTTPCaching preventHTTPCaching} on a
     * per-operationType basis.
     *
     * <p>Default value is null
     */
    public Boolean preventHTTPCaching;

    /**
     * The name of the method to invoke on the {@link com.smartgwt.client.docs.serverds.ServerObject} for this
     * operationBinding.
     *
     * <p>Default value is null
     */
    public String serverMethod;

    /**
     * For an operationBinding of {@link com.smartgwt.client.docs.serverds.OperationBinding#operationType 'operationType'}
     * "add" or "update", this  property is the {@link com.smartgwt.client.docs.serverds.OperationBinding#operationId
     * 'operationId'} of a "fetch" operationBinding to use  for cache synchronization purposes (ie, to fetch the row most
     * recently added or changed). This property, along with {@link
     * com.smartgwt.client.docs.serverds.OperationBinding#useForCacheSync 'useForCacheSync'} and  {@link
     * com.smartgwt.client.docs.serverds.OperationBinding#canSyncCache 'canSyncCache'} is provided so that you can use custom
     * database operations without sacrificing the benefits of Smart GWT's automatic cache synchronization. <p> This property
     * is only applicable to DataSources of type "sql".
     *
     * <p>Default value is null
     */
    public String cacheSyncOperation;

    /**
     * Specifies, for this specific operationBinding, whether to qualify column names with table names in any SQL we generate. 
     * Overrides the {@link com.smartgwt.client.docs.serverds.DataSource#qualifyColumnNames qualifyColumnNames} property. Only
     * applicable to dataSources of {@link com.smartgwt.client.docs.serverds.DataSource#serverType 'serverType'} "sql".
     *
     * <p>Default value is true
     */
    public Boolean qualifyColumnNames;

    /**
     * For a dataSource of {@link com.smartgwt.client.docs.serverds.DataSource#serverType 'serverType'} "hibernate", this 
     * property can be specified on an operationBinding to indicate that the server should run  user-specified HQL (Hibernate
     * Query Language), rather than the Hibernate criteria query or  <code>saveOrUpdate</code> call it would normally generate
     * to satisfy a dataSource operation via Hibernate.  <p> Note that inserting new records via HQL is often impractical, due
     * to intentional restrictions  in the language (it is only possible to perform an insert expressed in terms of a SELECT;
     * the "VALUES" construct commonly used when inserting new rows singly is not supported).  If you are  intending to use
     * customHQL, we recommend that you avoid doing so for {@link com.smartgwt.client.data.OperationBinding}s  with {@link
     * com.smartgwt.client.docs.serverds.OperationBinding#operationType 'operationType'} "add", unless you have a special 
     * requirement such as a bulk insert; if you need custom queries to perform inserts  on "hibernate" dataSources, we
     * recommend you use {@link com.smartgwt.client.docs.serverds.OperationBinding#customSQL 'customSQL'}, which is valid for
     * "hibernate" DataSources as well as "sql" dataSources. <p> For other operations on "hibernate" dataSources, however, HQL
     * has the advantage of  being more portable across different database engines than is plain SQL.
     *
     * <p>Default value is null
     */
    public String customHQL;

    /**
     * For an operationBinding of {@link com.smartgwt.client.docs.serverds.OperationBinding#operationType 'operationType'}
     * "fetch" which specifies no {@link com.smartgwt.client.docs.serverds.OperationBinding#operationId 'operationId'}, this
     * property determines whether the operationBinding  should be used for cache synchronization purposes (ie, to retrieve the
     * record most recently added or changed).  This property has no effect on an operationBinding that specifies an 
     * operationId - see {@link com.smartgwt.client.docs.serverds.OperationBinding#cacheSyncOperation cacheSyncOperation}. <p>
     * In order to work correctly with Smart GWT's cache synchronization system, an  operationBinding marked useForCacheSync
     * should have the following properties: <ul> <li>Able to complete its retrieval using no context other than the values of
     * the primary key fields declared in the dataSource (these will be provided in the $criteria object passed to  the
     * operation)</li> <li>Returns the entire record, including any values that may require joins to other tables or other
     * complexities</li> </ul> This property is only applicable to DataSources of type "sql".
     *
     * <p>Default value is true
     */
    public Boolean useForCacheSync;

    /**
     * For a dataSource of {@link com.smartgwt.client.docs.serverds.DataSource#serverType 'serverType'} "sql", this  property
     * can be specified on an operationBinding to provide the server with a bespoke WHERE clause to use when constructing the
     * SQL query to perform this operation.  The property should be a valid expression in the syntax of the underlying
     * database.  The  server will insert the text of this property immediately after the "WHERE" token. <p> You may find the
     * Smart GWT-provided <b>$criteria</b> variable of particular use with this property. <p> See the documentation for {@link
     * com.smartgwt.client.docs.serverds.OperationBinding#customSQL customSQL} for usage examples
     *
     * <p>Default value is null
     */
    public String whereClause;

    /**
     * For a dataSource of {@link com.smartgwt.client.docs.serverds.DataSource#serverType 'serverType'} "sql", this  property
     * can be specified on an operationBinding to provide the server with a bespoke set of values to add or update,  for use
     * when constructing the SQL query to perform this  operation.  The property should be one of the following, depending on
     * the  {@link com.smartgwt.client.docs.serverds.OperationBinding#operationType 'operationType'}: <p> For "add" operations,
     * the syntax that would be valid for an INSERT INTO query: a  comma-separated list of column names enclosed in
     * parentheses, followed by a comma-separated list of new values, enclosed in parentheses and preceded by the token
     * "VALUES".  For example: <p> <code>&lt;valuesClause&gt;(name, age) VALUES("Jane Doe", 48)&lt;/valuesClause&gt;</code> <p>
     * For "update" operations, the syntax that would be valid for an UPDATE query: a  comma-separated list of expressions
     * equating a column name to its new value.  For example: <p> <code>&lt;valuesClause&gt;name="Jane Doe",
     * age=48&lt;/valuesClause&gt;</code> <p> You may find the Smart GWT-provided <b>$values</b> variable of particular use
     * with this property. <p> See the documentation for {@link com.smartgwt.client.docs.serverds.OperationBinding#customSQL
     * customSQL} for usage examples
     *
     * <p>Default value is null
     */
    public String valuesClause;

    /**
     * Setting <code>useFlatFields</code> on an operationBinding is equivalent to setting {@link
     * com.smartgwt.client.data.DSRequest#getUseFlatFields useFlatFields} on all DataSource requests with the same {@link
     * com.smartgwt.client.docs.serverds.OperationBinding#operationType operationType} as this <code>operationBinding</code>.
     * <P> Typical usage is to combine operationBinding.useFlatFields with  {@link
     * com.smartgwt.client.widgets.DataBoundComponent#getUseFlatFields 'searchForm.useFlatFields'}, with the {@link
     * com.smartgwt.client.widgets.form.SearchForm} bound to the {@link com.smartgwt.client.data.WebService#getInputDS} of the
     * web service operation set as {@link com.smartgwt.client.data.OperationBinding#getWsOperation wsOperation}.  This allows
     * gratuitous nesting to be consistently bypassed in both the user presentation and in the actual XML messaging. <P> Note
     * that <code>useFlatFields</code> is not generally recommended for use with input messages where multiple simple type
     * fields exist with the same name, however if used in this way, the first field to use a given name wins.  "first" means
     * the first field encountered in a depth first search.  "wins" means only the first field will be available in data
     * binding, and only the first field will be populated in the generated XML message.
     *
     * <p>Default value is false
     */
    public Boolean useFlatFields;

    /**
     * The name of the file to save the exported data into.
     *
     * <p>Default value is null
     */
    public String exportFilename;

    /**
     * For a dataSource of {@link com.smartgwt.client.docs.serverds.DataSource#serverType 'serverType'} "sql", this  property
     * can be specified on an operationBinding to provide the server with a bespoke SELECT clause to use when constructing the
     * SQL query to perform this operation.  The property should be a comma-separated list of column names and/or expressions,
     * and you  can refer to any scalar function supported by the underlying database.  The server will  insert the text of
     * this property immediately after the "SELECT" token. <p> Note that if you also specify a {@link
     * com.smartgwt.client.docs.serverds.OperationBinding#groupClause 'groupClause'}, you can use aggregate  functions such as
     * SUM and COUNT in the selectClause. <p> This property is only applicable to operationBindings of  {@link
     * com.smartgwt.client.docs.serverds.OperationBinding#operationType 'operationType'} "fetch". <p> See the documentation for
     * {@link com.smartgwt.client.docs.serverds.OperationBinding#customSQL customSQL} for usage examples
     *
     * <p>Default value is null
     */
    public String selectClause;

    /**
     * For a dataSource of {@link com.smartgwt.client.docs.serverds.DataSource#serverType 'serverType'} "sql" or "hibernate",
     * this  property can be specified on an operationBinding to indicate that the server should run  user-specified SQL,
     * rather than the query it would normally generate to satisfy a  dataSource operation.  This property allows you provide a
     * fully-customized query; an alternative is to provide custom "pieces" to the query generator, via properties such as
     * {@link com.smartgwt.client.docs.serverds.OperationBinding#whereClause 'whereClause'} and {@link
     * com.smartgwt.client.docs.serverds.OperationBinding#valuesClause 'valuesClause'}.  See  {@link
     * com.smartgwt.client.docs.CustomQuerying} for more details. <p> For a dataSource of type "sql", the Smart GWT server
     * generates a number of useful query  "pieces" and makes them available to your custom SQL code via the Velocity
     * templating  language (note that this is not available for "hibernate" dataSources).  These are  documented {@link
     * com.smartgwt.client..DefaultQueryClause}. <p> We also make the template variables <b>$criteria</b> and <b>$values</b>
     * available, to give you direct access to the supplied criteria, and to the new field values for update and add
     * operations.  These variables are available to both "sql" and "hibernate" dataSources. <p> Note that you should use this
     * feature with care. In particular, writing customSQL code that makes use of a particular database engine's features or
     * syntax will make your  application less portable.  One way to avoid tying your query to a particular database's
     * name-quoting idiosyncracies (every database product seems to have a different idea about  how to treat unquoted names,
     * and which characters to use to quote names) is to use the  template variables <b>$fields</b> and <b>$qfields</b>, whioh
     * provide the names of your dataSource fields correctly quoted in the syntax of the underlying database. <p> See {@link
     * com.smartgwt.client.docs.CustomQuerying} for an overview of writing custom queries and clauses. <p> <h3>Examples</h3> An
     * example using the Smart GWT-supplied query pieces.  This custom query will give  exactly the same result as the Smart
     * GWT-generated query: <p> <code> &lt;operationBinding operationId="customFetch" operationType="fetch"&gt;<br>
     * &nbsp;&nbsp;&lt;customSQL&gt;<br> &nbsp;&nbsp;&nbsp;&nbsp;SELECT $defaultSelectClause FROM $defaultTablesClause
     * &nbsp;&nbsp;&nbsp;&nbsp;WHERE $defaultWhereClause ORDER BY $defaultOrderClause &nbsp;&nbsp;&lt;/customSQL&gt;<br>
     * &lt;/operationBinding&gt;<br> </code> <p> An example using the Smart GWT-supplied <b>$criteria</b> template variable:
     * <p> <code> &lt;operationBinding operationId="customFetch" operationType="fetch"&gt;<br>
     * &nbsp;&nbsp;&lt;customSQL&gt;<br> &nbsp;&nbsp;&nbsp;&nbsp;SELECT foo, bar, baz FROM MyTable WHERE bar >
     * $criteria.someValue<br> &nbsp;&nbsp;&lt;/customSQL&gt;<br> &lt;/operationBinding&gt;<br> </code> <p> An update example:
     * <p> <code> &lt;operationBinding operationId="myUpdateOp" operationType="update"&gt;<br>
     * &nbsp;&nbsp;&lt;customSQL&gt;<br> &nbsp;&nbsp;&nbsp;&nbsp;UPDATE $defaultTableClause SET $defaultValuesClause WHERE bar 
     * &lt;= $criteria.someValue<br> &nbsp;&nbsp;&lt;/customSQL&gt;<br> &lt;/operationBinding&gt;<br> </code>
     *
     * <p>Default value is null
     */
    public String customSQL;

    /**
     * You can explicitly declare the arguments to be passed to {@link
     * com.smartgwt.client.docs.serverds.OperationBinding#serverMethod serverMethod} using this attribute.  This isn't required
     * - in the absense of <code>methodArguments</code>, the DMI implementation will still automatically pass a stock set of
     * arguments to your method (see the overview in {@link com.smartgwt.client.docs.serverds.ServerObject}), but specifying
     * arguments explicitly gives you more flexibility in what can be passed. <p> The format for specifying
     * <code>methodArguments</code> is as a comma separated list of VTL (Velocity Template Language) expressions.  See the <a
     * href='http://jakarta.apache.org/velocity/docs/vtl-reference-guide.html'
     * onclick="window.open('http://jakarta.apache.org/velocity/docs/vtl-reference-guide.html');return false;">VTL
     * Reference</a> and <a href='http://jakarta.apache.org/velocity/docs/user-guide.html'
     * onclick="window.open('http://jakarta.apache.org/velocity/docs/user-guide.html');return false;">Velocity User Guide</a> 
     * for an overview of how to use VTL. <p> The Velocity context is pre-populated with the following variables - you can pass
     * these verbatim as arguments, or call methods on these objects and pass the resulting values: <ul> <li>dsRequest:
     * instance of the current DSRequest <li>request: the current HttpServletRequest <li>response: the current
     * HttpServletResponse <li>rpcManager: the instance of RPCManager for this request <li>dataSource: a DataSource instance
     * for this request </ul> So, for example, if you had a method signature like the following: <p><code> public DSResponse
     * fetch(SupplyItem criteria, long startRow, long endRow) </code><p> You can invoke it by specifying
     * <code>methodArguments</code> as follows: <p><code> methodArguments="$dsRequest.criteria, $dsRequest.startRow,
     * $dsRequest.endRow" </code></p> Without <code>methodArguments</code>, there would be no way for you to specify
     * <code>startRow/endRow</code> as arguments.  You could, of course, simply declare the method to take a
     * <code>DSRequest</code> object and call <code>getStartRow()/getEndRow()</code> in the body of the method.
     *
     * <p>Default value is null
     */
    public String methodArguments;

    /**
     * For a dataSource of {@link com.smartgwt.client.docs.serverds.DataSource#serverType 'serverType'} "sql", this  property
     * can be specified on an operationBinding to provide the server with a bespoke GROUP BY clause to use when constructing
     * the SQL query to perform this operation.  The property should be a comma-separated list of column names and/or
     * expressions, forming a valid GROUP BY clause in the syntax of the underlying database.  The server will  insert the text
     * of this property immediately after the "GROUP BY" token. <p> Note that specifying this property enables you to use
     * aggregate functions (such as  COUNT and SUM) in your {@link
     * com.smartgwt.client.docs.serverds.OperationBinding#selectClause 'selectClause'}. Also note that care is required  when
     * using groupClause to ensure that the selectClause contains the fields you are grouping by.  Failure to do this correctly
     * will result in a runtime SQL error. <p> This property is only applicable to operationBindings of  {@link
     * com.smartgwt.client.docs.serverds.OperationBinding#operationType 'operationType'} "fetch". <p> See the documentation for
     * {@link com.smartgwt.client.docs.serverds.OperationBinding#customSQL customSQL} for usage examples
     *
     * <p>Default value is null
     */
    public String groupClause;
             
    /**
     * Controls the format in which inputs are sent to the dataURL. <p> When a DataSource operation such as fetchData() is
     * invoked on this DataSource or a component bound to this DataSource, the data passed to the operation, if any, will be
     * sent to the <code>dataURL</code>.  The <code>dataProtocol</code> property controls the format in which the data is sent:
     * SOAP message, HTTP GET or POST of parameters, etc. <P> The <code>dataProtocol</code> property need not be set for a
     * DataSource with a WebService ( {@link com.smartgwt.client.docs.serverds.DataSource#serviceNamespace serviceNamespace} is
     * set), in this case, SOAP messaging is used by default. <P> Developers may completely bypass the Smart GWT comm system by
     * setting dataProtocol to <code>"clientCustom"</code>. In this case Smart GWT will not attempt to send any data to the
     * server after calling {@link com.smartgwt.client.data.DataSource#transformRequest}. Instead the developer is expected to
     * implement <code>transformRequest()</code> such that it performs the necessary data action outside of Smart GWT, and then
     * calls {@link com.smartgwt.client.data.DataSource#processResponse},  passing in the {@link
     * com.smartgwt.client.data.DSRequest#getRequestId requestId} and an appropriate set of DSResponse properties to indicate
     * the result of the action. <P> NOTE: when {@link com.smartgwt.client.data.OperationBinding#getDataFormat dataFormat} is
     * "iscServer", <code>dataProtocol</code> is not consulted. Instead, Smart GWT uses a proprietary wire format to
     * communicate with the Smart GWT server, and the server-side DSRequest and DSResponse objects should be used to access
     * request data and form responses.
     *
     * <p>Default value is "getParams"
     */
    public DSProtocol dataProtocol;

    /**
     * For an operationBinding of {@link com.smartgwt.client.docs.serverds.OperationBinding#operationType 'operationType'}
     * "add" or "update", this  property indicates to the server that cache synchronization will work after this operation has
     * performed its updates, without the need for an explicit  {@link
     * com.smartgwt.client.docs.serverds.OperationBinding#cacheSyncOperation 'cacheSyncOperation'}.  This property tells the
     * server that the  default cache sync operation (either a custom one declared with no operationId, as described in the
     * docs for {@link com.smartgwt.client.docs.serverds.OperationBinding#useForCacheSync useForCacheSync}, or the default one
     * that Smart GWT generates) will  return the correct record to the client, so the client's cache does not need to be 
     * invalidated. <p> This property is only applicable to DataSources of type "sql".
     *
     * <p>Default value is null
     */
    public Boolean canSyncCache;

    /**
     * For an XML or JSON DataSource, XPath expression used to retrieve the objects that will become DataSource records. <p>
     * For example, an "ItemSearch" web service might return a "Results" structure containing metadata along with the set of
     * Items that one might want to display in a grid.  An XPath expression like "/Results/Items" could be used to retrieve
     * just the Items, which would then become DataSource records. <p> For a JSON web service, the <code>recordXPath</code> is
     * applied to the returned JSON data via {@link com.smartgwt.client.data.XMLTools#selectObjects}.  Only limited XPath
     * syntax is allowed; see  {@link com.smartgwt.client.data.XMLTools#selectObjects} for details. <P> For processing XML
     * results, see {@link com.smartgwt.client.docs.serverds.OperationBinding#xmlNamespaces xmlNamespaces} for information on
     * the namespaces that are available in this XPath expression.  If you are contacting a WSDL web service, note that {@link
     * com.smartgwt.client.docs.serverds.OperationBinding#recordName recordName} is an alternative way to specify which records
     * should be selected by their tagName or type, and this is usually simpler. <P> To learn about XPath, try the following
     * search: <a href="http://www.google.com/search?q=xpath+tutorial" target="_blank"
     * >http://www.google.com/search?q=xpath+tutorial</a>
     *
     * <p>Default value is null
     */
    public String recordXPath;

    /**
     * Comma-separated list of user roles that are allowed to invoke the operation described by&#010 this
     * operationBinding.&#010 <P>&#010 Whether the current user has a given role is determined by calling the standard
     * Java&#010 servlets method <code>httpServletRequest.isUserInRole()</code>, hence works with both simple&#010 J2EE
     * security (realms and form-based authentication) and JAAS (Java Authentication &&#010 Authorization Service).&#010
     * <P>&#010 If you wish to use a role-based security scheme that does not make use of the servlet API's &#010 standards,
     * Smart GWT Server also implements the <code>setUserRoles</code> method&#010 on <code>RPCManager</code>.  You can use this
     * API to tell Smart GWT that all the &#010 requests in the queue currently being processed are associated with a user who
     * has the roles&#010 you supply; in this case, Smart GWT will not attempt to resolve the user's roles via &#010
     * <code>httpServletRequest.isUserInRole()</code>.&#010 <P>&#010 If there an operationBinding declared for a given
     * operationType which does not have an&#010 {@link com.smartgwt.client.docs.serverds.OperationBinding#operationId
     * operationId}, that is, it is the default operationBinding for the type, then any other&#010 operationBinding of the same
     * type is assumed to have the same setting for&#010 <code>requiresRole</code> as the default operationBinding for the
     * operationType.  For&#010 example, given these declarations:&#010 <pre>&#010     &lt;operationBinding
     * operationType="fetch" requiresRole="manager"&gt;&#010           ... settings ...&#010     
     * &lt;/operationBinding&gt;&#010     &lt;operationBinding operationType="fetch"
     * operationId="fetchWithExtraFields"&gt;&#010           ... settings ...&#010      &lt;/operationBinding&gt;&#010
     * </pre>&#010 The second operationBinding requires the "manager" role even though there is no explicit&#010
     * <code>requiresRole</code> declaration.  To prevent the "manager" role being required by the&#010 second
     * operationBinding, add <code>requireRole=""</code>.&#010 <P>&#010 Note that if {@link
     * com.smartgwt.client.data.DataSource#getRequiresRole requiresRole} is set, all operations on the DataSource require&#010
     * the roles set for the DataSource as a whole, even if they declare individual&#010 <code>requiresRole</code>
     * attributes.&#010 <P>&#010 This property is valid only for a server-side DataSource when using the Smart GWT Server.
     *
     * <p>Default value is null
     */
    public String requiresRole;

    /**
     * URL to contact to fulfill DSRequests for this operationBinding. <P> <code>dataURL</code> is typically set as
     * DataSource.dataURL rather than on each individual operationBinding. <P> <code>dataURL</code> can be omitted for a
     * DataSource using a Web Service ({@link com.smartgwt.client.docs.serverds.DataSource#serviceNamespace serviceNamespace}
     * is set).
     *
     * <p>Default value is null
     */
    public String dataURL;

    /**
     * Whether a user must be authenticated in order to access this operation.  For details of  what is meant by
     * "authenticated", see {@link com.smartgwt.client.data.DataSource#getRequiresAuthentication requiresAuthentication}. <P>
     * To protect access to an entire operationType (eg, all "fetch" operations), declare an operationBinding with
     * <code>requiresAuthentication="true"</code>, {@link com.smartgwt.client.docs.serverds.OperationBinding#operationType
     * operationType} set to the operationType to be protected, but no {@link
     * com.smartgwt.client.docs.serverds.OperationBinding#operationId operationId}.  This will then prevent access to the
     * "fetch" operationType unless another {@link com.smartgwt.client.data.OperationBinding} declares
     * requiresAuthentication="false" with a specific {@link com.smartgwt.client.docs.serverds.OperationBinding#operationId
     * 'operationId'}.
     *
     * <p>Default value is null
     */
    public Boolean requiresAuthentication;

    /**
     * Optional operationId if this DataSource supports two or more variants of one of the basic DataSource operations, for
     * instance, a "fetch" that uses full text search and a "fetch" that accepts per-field search criteria.  See {@link
     * com.smartgwt.client.data.DSRequest#getOperationId operationId} for usage.
     *
     * <p>Default value is null
     */
    public String operationId;

    /**
     * When set, causes the results of the DataSource Operation to be exported to a file, whose &#010 name and format are
     * indicated by {@link com.smartgwt.client.docs.serverds.OperationBinding#exportFilename exportFilename} and &#010 {@link
     * com.smartgwt.client.docs.serverds.OperationBinding#exportAs exportAs} respectively.  When no exportFilename is provided,
     * the &#010 default is <i>Results</i> and the default value of exportAs is <i>csv</i>.  Once the Operation &#010
     * completes, {@link com.smartgwt.client.data.DSRequest#getExportDisplay exportDisplay} specifies whether the &#010
     * exported data will be downloaded to the file-system or displayed in a new window.   The &#010 default value of
     * exportDisplay is "download" which displays the Save As dialog.  See &#010 {@link
     * com.smartgwt.client.types.ExportDisplay} for more information.&#010 <P>&#010 The export field-list can also be
     * configured, see {@link com.smartgwt.client.data.DSRequest#getExportFields exportFields}.&#010 <P>&#010 You can also
     * configure the style of line-breaks to use when generating the output.  See&#010 {@link
     * com.smartgwt.client..LineBreakStyle} for more information.&#010 <P>&#010 As well as setting this and other properties on
     * the {@link com.smartgwt.client.data.OperationBinding}, Exports can be &#010 initiated in two other ways.  You can set
     * properties on the dsRequest by passing &#010 <i>requestProperties</i> into {@link
     * com.smartgwt.client.data.DataSource#exportData}.  Note that this method does&#010 not support exporting to JSON format
     * (see &#010 <a href="http://forums.smartclient.com/showthread.php?t=235">this post</a> for more detail).&#010
     * Additionally, custom server code may set export-related properties on the &#010 {@link
     * com.smartgwt.client.data.DSResponse}.&#010 <P>&#010 <b>Format Examples</b>&#010 <P>&#010 XML format&#010 <pre>&#010    
     * &lt;List&gt;&#010         &lt;Object&gt;&#010             &lt;id&gt;10101&lt;/id&gt;&#010            
     * &lt;displayName&gt;Record 10101&lt;/displayName&gt;&#010         &lt;/Object&gt;&#010    &lt;/List&gt;&#010 </pre>&#010
     * JSON Format&#010 <pre>&#010     [&#010         { id: 10101, displayName: "Record 10101" }&#010     ]&#010 </pre>&#010
     * CSV Format&#010 <pre>&#010     id,displayName&#010     10101,"Record 10101"&#010 </pre>
     *
     * <p>Default value is false
     */
    public Boolean exportResults;

    /**
     * For an XML DataSource, tagName of the elements to be used as records. <p> This is a simple alternative to {@link
     * com.smartgwt.client.docs.serverds.OperationBinding#recordXPath recordXPath} when the elements to be used as records all
     * share a tagName. <p> When a DataSource has a WebService, <code>recordName</code> can also be set to the name of any
     * <code>complexType</code> declared within the WebService's WSDL file.
     *
     * <p>Default value is null
     */
    public String recordName;

    /**
     * For a dataSource of {@link com.smartgwt.client.docs.serverds.DataSource#serverType 'serverType'} "sql", this  property
     * can be specified on an operationBinding to provide the server with a bespoke table clause to use when constructing the
     * SQL query to perform this operation.  The property should be a comma-separated list of tables and views, and you can use
     * any special language constructs supported by the underlying database.  The server will  insert the text of this property
     * immediately after the "FROM" token. <p> See the documentation for {@link
     * com.smartgwt.client.docs.serverds.OperationBinding#customSQL customSQL} for usage examples
     *
     * <p>Default value is null
     */
    public String tableClause;

    /**
     * For a dataSource of {@link com.smartgwt.client.docs.serverds.DataSource#serverType 'serverType'} "sql", this  property
     * can be specified on an operationBinding to provide the server with a bespoke ORDER BY clause to use when constructing
     * the SQL query to perform this operation.  The property should be a comma-separated list of column names and/or
     * expressions, forming a valid ORDER BY clause in the syntax of the underlying database.  The server will  insert the text
     * of this property immediately after the "ORDER BY" token. <p> This property is only applicable to operationBindings of 
     * {@link com.smartgwt.client.docs.serverds.OperationBinding#operationType 'operationType'} "fetch". <p> See the
     * documentation for {@link com.smartgwt.client.docs.serverds.OperationBinding#customSQL customSQL} for usage examples
     *
     * <p>Default value is null
     */
    public String orderClause;


    /**
     * Specifies, for this operationBinding only, a list of extra field names that should be included in the default {@link
     * com.smartgwt.client.docs.serverds.OperationBinding#selectClause 'selectClause'} Smart GWT generates.  Here, "extra" refers to
     * fields that would not normally be included because they are marked as  {@link
     * com.smartgwt.client.docs.serverds.DataSourceField#customSQL 'customSQL'}. You can specify this property as either an array or
     * a string containing a comma-separated list  of field names (eg, "foo, bar, baz"). <p> This property is only applicable
     * to DataSources of type "sql".
     *
     */
    public String[] customValueFields;

    /**
     * Specifies, for this operationBinding only, a list of field names that should be excluded from the default {@link
     * com.smartgwt.client.docs.serverds.OperationBinding#whereClause 'whereClause'} Smart GWT generates.  The idea behind this is
     * that you can then use these criteria manually in a complex query - for example, in the  WHERE clause of a subquery. <p>
     * You can specify this property as either an array or a string containing a comma-separated list  of field names (eg,
     * "foo, bar, baz").  Note that if a field is included in both  excludeCriteriaFields and {@link
     * com.smartgwt.client.docs.serverds.OperationBinding#customCriteriaFields 'customCriteriaFields'},  customCriteriaFields wins.
     * <p> This property is only applicable to DataSources of type "sql".
     *
     */
    public String[] excludeCriteriaFields;

    /**
     * A list of ${isc.DocUtils.linkForRef('object:DSRequestModifier')}s that will be used to modify the critieria of each
     * {@link com.smartgwt.client.data.DSRequest} that uses this operationBinding.
     *
     */
    public DSRequest[]  criteria;

    /**
     * The list of field-names to export.  If provided, the field-list in the exported output is  limited and sorted as per the
     * list. <P> If exportFields is not provided, the exported output includes all visible fields  from the DataSource
     * (field.hidden=false), sorted in the order they're defined.
     *
     */
    public String[] exportFields;


    /**
     * A list of ${isc.DocUtils.linkForRef('object:DSRequestModifier')}s that will be used to modify the values object of each
     * {@link com.smartgwt.client.data.DSRequest} that uses this operationBinding.
     *
     */
    public DSRequest[] values;


    /**
     * Optional object declaring namespace prefixes for use in {@link com.smartgwt.client.data.OperationBinding#setRecordXPath
     * recordXPath} and&#010 {@link com.smartgwt.client.docs.serverds.DataSourceField#valueXPath} XPath expressions.&#010
     * <P>&#010 <code>xmlNamespaces</code> should be specified as a mapping from namespace prefix to&#010 namespace URI, for
     * example:&#010 <pre>&#010    xmlNamespaces : {&#010        az :
     * "http://webservices.amazon.com/AWSECommerceService/2005-03-23"&#010    }&#010 </pre>&#010 By default, all namespaces
     * declared on the document element (outermost element of the&#010 response) are made available with the prefix used in the
     * document itself.&#010 <P>&#010 Then, for non-WSDL-described XML results, if there is a default namespace on the
     * document&#010 element, it is made available with the special prefix "default".&#010 <P>&#010 For results of
     * WSDL-described operations, the prefix "service" means the service namespace,&#010 that is, the "targetNamespace" on the
     * &lt;definitions&gt; element from the WSDL file.  The prefix&#010 "schema" means the namespace of the outermost element
     * in the output message for the current&#010 operation.  "default" will be the schema namespace if there is one, otherwise
     * the service&#010 namespace.&#010 <P>&#010 For basic information on XML Namespaces and their use in XPath, try the
     * following search:&#010 <a href="http://www.google.com/search?q=XPath+xml+namespaces" target="_blank"&#010
     * >http://www.google.com/search?q=XPath+xml+namespaces</a>
     *
     */
    public Map xmlNamespaces;

    /**
     * Specifies, for this operationBinding only, a list of extra field names that should be included in the default {@link
     * com.smartgwt.client.docs.serverds.OperationBinding#whereClause 'whereClause'} Smart GWT generates.  Here, "extra" refers to
     * fields that would not normally be included because they are marked as  {@link
     * com.smartgwt.client.docs.serverds.DataSourceField#customSQL 'customSQL'}. You can specify this property as either an array or
     * a string containing a comma-separated list  of field names (eg, "foo, bar, baz"). <p> This property is only applicable
     * to DataSources of type "sql".
     *
     */
    public String[] customCriteriaFields;

    /**
     * HTTP parameters that should be submitted with every DSRequest. <P> Useful for authenticated services that require a
     * sessionId with every request. <P> Can be set for all operations of a given DataSource as DataSource.defaultParams.
     *
     */
    public Map defaultParams;

    /**
     * Additional properties to pass through to the {@link com.smartgwt.client.data.DSRequest} created for this operation. Note
     * that these will be cumulative with and will override on a per-property basis any properties set via {@link
     * com.smartgwt.client.docs.serverds.DataSource#requestProperties}. <p> These properties are applied before
     * {@link com.smartgwt.client.data.DataSource#transformRequest} is called.
     *
     */
    public DSRequest requestProperties;

    /**
     * Optional ServerObject declaration that specifies the ServerObject configuration for this operationBinding.  In the
     * absense of a serverObject specification here, the one specified on the DataSource itself is used by default, if present
     * ({@link com.smartgwt.client.docs.serverds.DataSource#serverObject}).  If neither is present, then Direct
     * Method Invocation will not be enabled for this operationBinding.
     *
     */
    public ServerObject serverObject;

}
